package com.aitrainer.api.controller.diet

import com.aitrainer.api.model.diet.Recipe
import com.aitrainer.api.repository.diet.RecipeRepository
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*

@RestController
@RequestMapping("api")
class RecipeController(private val recipeRepository: RecipeRepository) {

    @PostMapping ("/recipe")
    fun insert(@RequestBody recipe: Recipe): ResponseEntity<Recipe> {
        val newRecipe = recipeRepository.save(recipe)
        return ResponseEntity.ok().body(newRecipe)
    }

    @PostMapping("/recipe/{id}")
    fun update(@PathVariable(value = "id") id: Long, @RequestBody recipe: Recipe): ResponseEntity<Recipe> {
        val existingRecipe = recipeRepository.findByRecipeId(id) ?: return ResponseEntity.notFound().build()

        val updatedRecipe: Recipe = existingRecipe.copy(
            name = recipe.name,
            description = recipe.description,
            cal = recipe.cal,
            ch = recipe.ch,
            fat = recipe.fat,
            protein = recipe.protein,
            dietUserId = recipe.dietUserId,
        )
        recipe.meals.forEach {
            it.recipe = recipe
            updatedRecipe.meals.add(it)
        }
        return ResponseEntity.ok().body(recipeRepository.save(updatedRecipe))
    }

    @GetMapping("/recipe")
    fun getAll(): ResponseEntity<List<Recipe>> {
        val list = recipeRepository.findAll()
        return if (list.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(list)
    }

    @GetMapping("/recipe/user/{dietUserId}")
    fun getByDietUserId(@PathVariable dietUserId: Long): ResponseEntity<List<Recipe>> {
        val list = recipeRepository.findByDietUserId(dietUserId)
        return if (list == null) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(list)
    }

    @GetMapping("/recipe/name/{name}")
    fun getByName(@PathVariable name: String): ResponseEntity<List<Recipe>> {
        val list = recipeRepository.findByName(name)
        return if (list == null) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(list)
    }
}